@@ -1,9 +1,9 @@
 /*
- * "$Id: file.c 1461 2005-11-03 21:11:16Z mike $"
+ * "$Id: file.c 1488 2006-05-31 19:00:02Z mike $"
  *
  *   Filename routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -38,6 +38,8 @@
  *   file_nolocal()     - Disable access to local files.
  *   file_proxy()       - Set the proxy host for all HTTP requests.
  *   file_referer()     - Set the HTTP referer for remote accesses.
+ *   file_rlookup()     - Lookup a filename to find the original URL, if
+ *                        applicable.
  *   file_target()      - Return the target of a link.
  *   file_temp()        - Create and open a temporary file.
  */
@@ -290,23 +292,21 @@ file_directory(const char *s)	/* I - Filename or URL */
     * Handle URLs...
     */
 
-    char	method[HTTP_MAX_URI],
+    char	scheme[HTTP_MAX_URI],
 		username[HTTP_MAX_URI],
 		hostname[HTTP_MAX_URI],
 		resource[HTTP_MAX_URI];
     int		port;
 
 
-    httpSeparate(s, method, username, hostname, &port, resource);
+    httpSeparateURI(HTTP_URI_CODING_ALL, s, scheme, sizeof(scheme),
+                    username, sizeof(username), hostname, sizeof(hostname),
+		    &port, resource, sizeof(resource));
     if ((dir = strrchr(resource, '/')) != NULL)
       *dir = '\0';
 
-    if (username[0])
-      snprintf(buf, sizeof(buf), "%s://%s@%s:%d%s", method, username, hostname,
-               port, resource);
-    else
-      snprintf(buf, sizeof(buf), "%s://%s:%d%s", method, hostname, port,
-               resource);
+    httpAssembleURI(HTTP_URI_CODING_ALL, buf, sizeof(buf), scheme, username,
+                    hostname, port, resource);
   }
   else
   {
@@ -381,7 +381,7 @@ file_find_check(const char *filename)	/* I - File or URL */
 {
   int		i;			/* Looping var */
   int		retry;			/* Current retry */
-  char		method[HTTP_MAX_URI],	/* Method/scheme */
+  char		scheme[HTTP_MAX_URI],	/* Method/scheme */
 		username[HTTP_MAX_URI],	/* Username:password */
 		hostname[HTTP_MAX_URI],	/* Hostname */
 		resource[HTTP_MAX_URI];	/* Resource */
@@ -392,24 +392,24 @@ file_find_check(const char *filename)	/* I - File or URL */
 		connauth[HTTP_MAX_VALUE];/* Auth string */
   http_status_t	status;			/* Status of request... */
   FILE		*fp;			/* Web file */
-  int		bytes,			/* Bytes read */
-		count,			/* Number of bytes so far */
-		total;			/* Total bytes in file */
+  ssize_t	bytes,			/* Bytes read */
+		count;			/* Number of bytes so far */
+  off_t		total;			/* Total bytes in file */
   char		tempname[HTTP_MAX_URI];	/* Temporary filename */
 
 
   DEBUG_printf(("file_find_check(filename=\"%s\")\n", filename));
 
   if (strncmp(filename, "http:", 5) == 0 || strncmp(filename, "//", 2) == 0)
-    strcpy(method, "http");
+    strcpy(scheme, "http");
 #ifdef HAVE_SSL
   else if (strncmp(filename, "https:", 6) == 0)
-    strcpy(method, "https");
+    strcpy(scheme, "https");
 #endif /* HAVE_SSL */
   else
-    strcpy(method, "file");
+    strcpy(scheme, "file");
 
-  if (strcmp(method, "file") == 0)
+  if (strcmp(scheme, "file") == 0)
   {
    /*
     * Return immediately if we aren't allowing access to local files...
@@ -443,7 +443,9 @@ file_find_check(const char *filename)	/* I - File or URL */
         return (web_cache[i].name);
       }
 
-    httpSeparate(filename, method, username, hostname, &port, resource);
+    httpSeparateURI(HTTP_URI_CODING_ALL, filename, scheme, sizeof(scheme),
+                    username, sizeof(username), hostname, sizeof(hostname),
+		    &port, resource, sizeof(resource));
 
     for (status = HTTP_ERROR, retry = 0;
          status == HTTP_ERROR && retry < 5;
@@ -457,7 +459,7 @@ file_find_check(const char *filename)	/* I - File or URL */
 
         connhost = proxy_host;
         connport = proxy_port;
-        snprintf(connpath, sizeof(connpath), "%s://%s:%d%s", method,
+        snprintf(connpath, sizeof(connpath), "%s://%s:%d%s", scheme,
                  hostname, port, resource);
       }
       else
@@ -482,7 +484,7 @@ file_find_check(const char *filename)	/* I - File or URL */
         progress_show("Connecting to %s...", connhost);
 
 #ifdef HAVE_SSL
-        if (strcmp(method, "http") == 0)
+        if (strcmp(scheme, "http") == 0)
           http = httpConnect(connhost, connport);
 	else
           http = httpConnectEncrypt(connhost, connport, HTTP_ENCRYPT_ALWAYS);
@@ -503,18 +505,19 @@ file_find_check(const char *filename)	/* I - File or URL */
 
       httpClearFields(http);
       httpSetField(http, HTTP_FIELD_HOST, hostname);
-      httpSetField(http, HTTP_FIELD_USER_AGENT, "HTMLDOC v" SVERSION);
       httpSetField(http, HTTP_FIELD_CONNECTION, "Keep-Alive");
       httpSetField(http, HTTP_FIELD_REFERER, referer_url);
 
       if (username[0])
       {
         strcpy(connauth, "Basic ");
-        httpEncode64(connauth + 6, username);
+        httpEncode64_2(connauth + 6, sizeof(connauth) - 6, username,
+	               strlen(username));
         httpSetField(http, HTTP_FIELD_AUTHORIZATION, connauth);
       }
 
-      httpSetCookie(http, cookies);
+      if (cookies[0])
+        httpSetCookie(http, cookies);
 
       if (!httpGet(http, connpath))
       {
@@ -535,8 +538,12 @@ file_find_check(const char *filename)	/* I - File or URL */
         * Grab new location from HTTP data...
 	*/
 
-        httpSeparate(httpGetField(http, HTTP_FIELD_LOCATION), method, username,
-	             hostname, &port, resource);
+	httpSeparateURI(HTTP_URI_CODING_ALL,
+	                httpGetField(http, HTTP_FIELD_LOCATION),
+			scheme, sizeof(scheme), username, sizeof(username),
+			hostname, sizeof(hostname), &port,
+			resource, sizeof(resource));
+
         status = HTTP_ERROR;
       }
     }
@@ -544,7 +551,7 @@ file_find_check(const char *filename)	/* I - File or URL */
     if (status != HTTP_OK)
     {
       progress_hide();
-      progress_error((HDerror)status, "%s", httpStatus(status));
+      progress_error((HDerror)status, "%s (%s)", httpStatus(status), filename);
       httpFlush(http);
       return (NULL);
     }
@@ -559,15 +566,15 @@ file_find_check(const char *filename)	/* I - File or URL */
       return (NULL);
     }
 
-    if ((total = atoi(httpGetField(http, HTTP_FIELD_CONTENT_LENGTH))) == 0)
+    if ((total = httpGetLength2(http)) == 0)
       total = 8192;
 
     count = 0;
-    while ((bytes = httpRead(http, resource, sizeof(resource))) > 0)
+    while ((bytes = httpRead2(http, resource, sizeof(resource))) > 0)
     {
       count += bytes;
       progress_update((100 * count / total) % 101);
-      fwrite(resource, 1, (size_t)bytes, fp);
+      fwrite(resource, 1, bytes, fp);
     }
 
     progress_hide();
@@ -943,7 +950,7 @@ file_nolocal()
 void
 file_proxy(const char *url)	/* I - URL of proxy server */
 {
-   char	method[HTTP_MAX_URI],	/* Method name (must be HTTP) */
+   char	scheme[HTTP_MAX_URI],	/* Method name (must be HTTP) */
 	username[HTTP_MAX_URI],	/* Username:password information */
 	hostname[HTTP_MAX_URI],	/* Hostname */
 	resource[HTTP_MAX_URI];	/* Resource name */
@@ -957,9 +964,11 @@ file_proxy(const char *url)	/* I - URL of proxy server */
   }
   else
   {
-    httpSeparate(url, method, username, hostname, &port, resource);
+    httpSeparateURI(HTTP_URI_CODING_ALL, url, scheme, sizeof(scheme),
+                    username, sizeof(username), hostname, sizeof(hostname),
+		    &port, resource, sizeof(resource));
 
-    if (strcmp(method, "http") == 0)
+    if (strcmp(scheme, "http") == 0)
     {
       strlcpy(proxy_host, hostname, sizeof(proxy_host));
       proxy_port = port;
@@ -982,6 +991,25 @@ file_referer(const char *referer)	/* I - Referer URL */
 }
 
 
+/*
+ * 'file_rlookup()' - Lookup a filename to find the original URL, if applicable.
+ */
+
+const char *				/* O - URL or filename */
+file_rlookup(const char *filename)	/* I - Filename */
+{
+  int		i;			/* Looping var */
+  cache_t	*wc;			/* Current cache file */
+
+
+  for (i = web_files, wc = web_cache; i > 0; i --, wc ++)
+    if (!strcmp(wc->name, filename))
+      return (wc->url);
+
+  return (filename);
+}
+
+
 /*
  * 'file_target()' - Return the target of a link.
  */
@@ -1089,5 +1117,5 @@ file_temp(char *name,			/* O - Filename */
 
 
 /*
- * End of "$Id: file.c 1461 2005-11-03 21:11:16Z mike $".
+ * End of "$Id: file.c 1488 2006-05-31 19:00:02Z mike $".
  */
