@@ -1,5 +1,5 @@
 /*
- * "$Id: html.cxx 1439 2005-10-28 20:32:59Z mike $"
+ * "$Id: html.cxx 1486 2006-05-31 15:27:28Z mike $"
  *
  *   HTML exporting functions for HTMLDOC, a HTML document processing program.
  *
@@ -113,11 +113,11 @@ html_export(tree_t *document,	/* I - Document to export */
   if (OutputFiles)
   {
     if (LogoImage[0])
-      image_copy(LogoImage, OutputPath);
+      image_copy(LogoImage, file_find(LogoImage, Path), OutputPath);
 
     for (int hfi = 0; hfi < MAX_HF_IMAGES; hfi ++)
       if (HFImage[hfi][0])
-        image_copy(HFImage[hfi], OutputPath);
+        image_copy(HFImage[hfi], file_find(HFImage[hfi], Path), OutputPath);
   }
 
   if (OutputFiles && TitleImage[0] && TitlePage &&
@@ -132,7 +132,7 @@ html_export(tree_t *document,	/* I - Document to export */
       strcmp(file_extension(TitleImage), "jpg") == 0 ||
       strcmp(file_extension(TitleImage), "png") == 0)
 #endif // WIN32
-    image_copy(TitleImage, OutputPath);
+    image_copy(TitleImage, file_find(TitleImage, Path), OutputPath);
 
  /*
   * Get document strings...
@@ -581,6 +581,7 @@ write_node(FILE   *out,		/* I - Output file */
   uchar		*ptr,		/* Pointer to output string */
 		*entity,	/* Entity string */
 		*src,		/* Source image */
+		*realsrc,	/* Real source image */
 		newsrc[1024];	/* New source image filename */
 
 
@@ -681,17 +682,18 @@ write_node(FILE   *out,		/* I - Output file */
 
     default :
 	if (t->markup == MARKUP_IMG && OutputFiles &&
-            (src = htmlGetVariable(t, (uchar *)"REALSRC")) != NULL)
+            (src = htmlGetVariable(t, (uchar *)"SRC")) != NULL &&
+            (realsrc = htmlGetVariable(t, (uchar *)"REALSRC")) != NULL)
 	{
 	 /*
-          * Update local images...
+          * Update and copy local images...
           */
 
           if (file_method((char *)src) == NULL &&
               src[0] != '/' && src[0] != '\\' &&
 	      (!isalpha(src[0]) || src[1] != ':'))
           {
-            image_copy((char *)src, OutputPath);
+            image_copy((char *)src, (char *)realsrc, OutputPath);
             strlcpy((char *)newsrc, file_basename((char *)src), sizeof(newsrc));
             htmlSetVariable(t, (uchar *)"SRC", newsrc);
           }
@@ -1113,5 +1115,5 @@ update_links(tree_t *t,		/* I - Document tree */
 
 
 /*
- * End of "$Id: html.cxx 1439 2005-10-28 20:32:59Z mike $".
+ * End of "$Id: html.cxx 1486 2006-05-31 15:27:28Z mike $".
  */
