@@ -1,9 +1,9 @@
 /*
- * "$Id: html.h 1457 2005-11-03 16:27:45Z mike $"
+ * "$Id: html.h 1479 2006-03-30 03:25:07Z mike $"
  *
  *   HTML parsing definitions for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -287,8 +287,10 @@ extern float		_htmlSizes[],
 			_htmlSpacings[];
 extern typeface_t	_htmlBodyFont,
 			_htmlHeadingFont;
+extern int		_htmlInitialized;
 extern char		_htmlCharSet[];
 extern float		_htmlWidths[TYPE_MAX][STYLE_MAX][256];
+extern int		_htmlUnicode[];
 extern const char	*_htmlGlyphs[];
 extern const char	*_htmlGlyphsAll[];
 extern const char	*_htmlFonts[TYPE_MAX][STYLE_MAX];
@@ -334,5 +336,5 @@ extern void	htmlDebugStats(const char *title, tree_t *t);
 #endif /* !_HTML_H_ */
 
 /*
- * End of "$Id: html.h 1457 2005-11-03 16:27:45Z mike $".
+ * End of "$Id: html.h 1479 2006-03-30 03:25:07Z mike $".
  */
