@@ -1,9 +1,9 @@
 /*
- * "$Id: htmldoc.h,v 1.18.2.25 2004/06/15 01:09:09 mike Exp $"
+ * "$Id: htmldoc.h 1421 2005-08-07 11:51:33Z mike $"
  *
  *   Header file for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -138,6 +138,7 @@ enum	/* PDF document permissions */
 #endif /* _HTML_DOC_CXX_ */
 
 VAR int		Verbosity	VALUE(0);	/* Verbosity */
+VAR int		OverflowErrors	VALUE(0);	/* Show errors on overflow */
 VAR int		StrictHTML	VALUE(0);	/* Do strict HTML checking */
 VAR int		CGIMode		VALUE(0);	/* Running as CGI? */
 VAR int		Errors		VALUE(0);	/* Number of errors */
@@ -190,6 +191,7 @@ VAR style_t	HeadFootStyle	VALUE(STYLE_NORMAL);
 VAR float	HeadFootSize	VALUE(11.0f);	/* Size of header & footer */
 
 VAR char	*Header[3]	NULL3,		/* Header for regular pages */
+		*Header1[3]	NULL3,		/* Header for first pages */
 		*TocHeader[3]	NULL3,		/* Header for TOC pages */
 		*Footer[3]	NULL3,		/* Regular page footer */
 		*TocFooter[3]	NULL3,		/* Footer for TOC pages */
@@ -267,6 +269,7 @@ extern void	set_page_size(const char *size);
 
 extern void	prefs_load(void);
 extern void	prefs_save(void);
+extern void	prefs_set_paths(void);
 
 extern char	*format_number(int n, char f);
 
@@ -275,5 +278,5 @@ extern char	*format_number(int n, char f);
 #endif /* __cplusplus */
 
 /*
- * End of "$Id: htmldoc.h,v 1.18.2.25 2004/06/15 01:09:09 mike Exp $".
+ * End of "$Id: htmldoc.h 1421 2005-08-07 11:51:33Z mike $".
  */
