@@ -1,9 +1,9 @@
 /*
- * "$Id: htmllib.cxx 1466 2005-11-18 01:37:50Z mike $"
+ * "$Id: htmllib.cxx 1492 2006-06-07 17:43:52Z mike $"
  *
  *   HTML parsing routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -185,6 +185,7 @@ char		_htmlCharSet[256] = "iso-8859-1";
 					/* Character set name */
 float		_htmlWidths[TYPE_MAX][STYLE_MAX][256];
 					/* Character widths of fonts */
+int		_htmlUnicode[256];	/* Character to Unicode mapping */
 const char	*_htmlGlyphsAll[65536];	/* Character glyphs for Unicode */
 const char	*_htmlGlyphs[256];	/* Character glyphs for charset */
 int		_htmlNumSorted = 0;	/* Number of sorted glyphs */
@@ -1152,26 +1153,67 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
 
           if ((face = htmlGetVariable(t, (uchar *)"FACE")) != NULL)
           {
-            for (ptr = face; *ptr != '\0'; ptr ++)
-              *ptr = tolower(*ptr);
+	    char	font[255],	// Font name
+			*fontptr;	// Pointer into font name
 
-            if (strstr((char *)face, "serif") != NULL)
-              t->typeface = TYPE_SERIF;
-            else if (strstr((char *)face, "sans") != NULL)
-              t->typeface = TYPE_SANS_SERIF;
-            else if (strstr((char *)face, "mono") != NULL)
-              t->typeface = TYPE_MONOSPACE;
-            else if (strstr((char *)face, "arial") != NULL ||
-                strstr((char *)face, "helvetica") != NULL)
-              t->typeface = TYPE_HELVETICA;
-            else if (strstr((char *)face, "times") != NULL)
-              t->typeface = TYPE_TIMES;
-            else if (strstr((char *)face, "courier") != NULL)
-              t->typeface = TYPE_COURIER;
-	    else if (strstr((char *)face, "symbol") != NULL)
-              t->typeface = TYPE_SYMBOL;
-	    else if (strstr((char *)face, "dingbat") != NULL)
-              t->typeface = TYPE_DINGBATS;
+
+            for (ptr = face; *ptr;)
+	    {
+	      while (isspace(*ptr) || *ptr == ',')
+	        ptr ++;
+
+              if (!*ptr)
+	        break;
+
+	      for (fontptr = font; *ptr && *ptr != ',' && !isspace(*ptr); ptr ++)
+	        if (fontptr < (font + sizeof(font) - 1))
+		  *fontptr++ = *ptr;
+
+              *fontptr = '\0';
+
+              if (!strcasecmp(font, "serif"))
+	      {
+        	t->typeface = TYPE_SERIF;
+		break;
+	      }
+              else if (!strcasecmp(font, "sans-serif") ||
+	               !strcasecmp(font, "sans"))
+	      {
+        	t->typeface = TYPE_SANS_SERIF;
+		break;
+	      }
+              else if (!strcasecmp(font, "mono"))
+	      {
+        	t->typeface = TYPE_MONOSPACE;
+		break;
+	      }
+              else if (!strcasecmp(font, "arial") ||
+	               !strcasecmp(font, "helvetica"))
+              {
+        	t->typeface = TYPE_HELVETICA;
+		break;
+	      }
+              else if (!strcasecmp(font, "times"))
+	      {
+        	t->typeface = TYPE_TIMES;
+		break;
+	      }
+              else if (!strcasecmp(font, "courier"))
+	      {
+        	t->typeface = TYPE_COURIER;
+		break;
+	      }
+	      else if (!strcasecmp(font, "symbol"))
+	      {
+        	t->typeface = TYPE_SYMBOL;
+		break;
+	      }
+	      else if (!strcasecmp(font, "dingbat"))
+	      {
+        	t->typeface = TYPE_DINGBATS;
+		break;
+	      }
+	    }
           }
 
           if ((color = htmlGetVariable(t, (uchar *)"COLOR")) != NULL)
@@ -2227,7 +2269,7 @@ htmlSetBaseSize(float p,	/* I - Point size of paragraph font */
 void
 htmlSetCharSet(const char *cs)		/* I - Character set file to load */
 {
-  int		i, j;			/* Looping vars */
+  int		i;			/* Looping var */
   char		filename[1024];		/* Filenames */
   FILE		*fp;			/* Files */
   int		ch, unicode;		/* Character values */
@@ -2302,7 +2344,7 @@ htmlSetCharSet(const char *cs)		/* I - Character set file to load */
   * Build the glyph array...
   */
 
-  for (i = 0, j = 0; i < 256; i ++)
+  for (i = 0; i < 256; i ++)
   {
    /*
     * Add the glyph to the charset array...
@@ -2315,6 +2357,9 @@ htmlSetCharSet(const char *cs)		/* I - Character set file to load */
     }
     else
       _htmlGlyphs[i] = _htmlGlyphsAll[chars[i]];
+
+    if (_htmlGlyphs[i])
+      _htmlUnicode[i] = chars[i];
   }
 
   htmlLoadFontWidths();
@@ -2909,7 +2954,7 @@ compute_size(tree_t *t)		/* I - Tree entry */
 
 
   if (!_htmlInitialized)
-    htmlSetCharSet("8859-1");
+    htmlSetCharSet("iso-8859-1");
 
   if (t->markup == MARKUP_IMG)
   {
@@ -3073,19 +3118,20 @@ get_alignment(tree_t *t)	/* I - Tree entry */
 
   if (align != NULL)
   {
-    if (strcasecmp((char *)align, "left") == 0)
+    if (!strcasecmp((char *)align, "left"))
       t->halignment = ALIGN_LEFT;
-    else if (strcasecmp((char *)align, "center") == 0)
+    else if (!strcasecmp((char *)align, "center"))
       t->halignment = ALIGN_CENTER;
-    else if (strcasecmp((char *)align, "right") == 0)
+    else if (!strcasecmp((char *)align, "right"))
       t->halignment = ALIGN_RIGHT;
-    else if (strcasecmp((char *)align, "justify") == 0)
+    else if (!strcasecmp((char *)align, "justify"))
       t->halignment = ALIGN_JUSTIFY;
-    else if (strcasecmp((char *)align, "top") == 0)
+    else if (!strcasecmp((char *)align, "top"))
       t->valignment = ALIGN_TOP;
-    else if (strcasecmp((char *)align, "middle") == 0)
+    else if (!strcasecmp((char *)align, "middle") ||
+             !strcasecmp((char *)align, "absmiddle"))
       t->valignment = ALIGN_MIDDLE;
-    else if (strcasecmp((char *)align, "bottom") == 0)
+    else if (!strcasecmp((char *)align, "bottom"))
       t->valignment = ALIGN_BOTTOM;
   }
 
@@ -3094,13 +3140,13 @@ get_alignment(tree_t *t)	/* I - Tree entry */
 
   if (align != NULL)
   {
-    if (strcasecmp((char *)align, "top") == 0)
+    if (!strcasecmp((char *)align, "top"))
       t->valignment = ALIGN_TOP;
-    else if (strcasecmp((char *)align, "middle") == 0)
+    else if (!strcasecmp((char *)align, "middle"))
       t->valignment = ALIGN_MIDDLE;
-    else if (strcasecmp((char *)align, "center") == 0)
+    else if (!strcasecmp((char *)align, "center"))
       t->valignment = ALIGN_MIDDLE;
-    else if (strcasecmp((char *)align, "bottom") == 0)
+    else if (!strcasecmp((char *)align, "bottom"))
       t->valignment = ALIGN_BOTTOM;
   }
 
@@ -3117,6 +3163,8 @@ fix_filename(char *filename,		/* I - Original filename */
              char *base)		/* I - Base directory */
 {
   char		*slash;			/* Location of slash */
+  char		temp[1024],		/* Temporary filename */
+		*tempptr;		/* Pointer into filename */
   static char	newfilename[1024];	/* New filename */
 
 
@@ -3125,6 +3173,40 @@ fix_filename(char *filename,		/* I - Original filename */
   if (filename == NULL)
     return (NULL);
 
+  // Unescape filenames as needed...
+  if (strchr(filename, '%') && !strstr(filename, "//"))
+  {
+    for (tempptr = temp; *filename && tempptr < (temp + sizeof(temp) - 1);)
+    {
+      if (*filename == '%')
+      {
+        // Decode hex-escaped filename character...
+	filename ++;
+	if (isxdigit(filename[0] & 255) && isxdigit(filename[1] & 255))
+	{
+	  if (isdigit(filename[0] & 255))
+	    *tempptr = (filename[0] - '0') << 4;
+	  else
+	    *tempptr = (tolower(filename[0]) - 'a' + 10) << 4;
+
+	  if (isdigit(filename[1] & 255))
+	    *tempptr |= filename[1] - '0';
+	  else
+	    *tempptr |= tolower(filename[0]) - 'a' + 10;
+
+          tempptr ++;
+	  filename += 2;
+	}
+	else
+	  *tempptr++ = '%';
+      }
+      else
+        *tempptr++ = *filename++;
+    }
+
+    filename = temp;
+  }
+
   if (strcmp(base, ".") == 0 || strstr(filename, "//") != NULL)
     return (file_find(Path, filename));
 
@@ -3369,5 +3451,5 @@ htmlFixLinks(tree_t *doc,		// I - Top node
 
 
 /*
- * End of "$Id: htmllib.cxx 1466 2005-11-18 01:37:50Z mike $".
+ * End of "$Id: htmllib.cxx 1492 2006-06-07 17:43:52Z mike $".
  */
