@@ -1,9 +1,9 @@
 /*
- * "$Id: http-private.h 1439 2005-10-28 20:32:59Z mike $"
+ * "$Id: http-private.h 1493 2006-06-07 18:20:49Z mike $"
  *
  *   Private HTTP definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -43,14 +43,14 @@
 #    define closesocket(f) close(f)
 #  endif /* WIN32 */
 
-#  ifdef __sgi
+#  if defined(__sgi) || (defined(__APPLE__) && !defined(_SOCKLEN_T))
 /*
- * IRIX does not define socklen_t, and in fact uses an int instead of
+ * IRIX and MacOS X 10.2.x do not define socklen_t, and in fact use an int instead of
  * unsigned type for length values...
  */
 
 typedef int socklen_t;
-#  endif /* __sgi */
+#  endif /* __sgi || (__APPLE__ && !_SOCKLEN_T) */
 
 #  include "http.h"
 
@@ -86,7 +86,20 @@ typedef struct
 
 #    include <Security/SecureTransport.h>
 
-typedef SSLConnectionRef http_tls_t;
+typedef struct				/**** CDSA connection information ****/
+{
+  SSLContextRef		session;	/* CDSA session object */
+  CFArrayRef		certsArray;	/* Certificates array */
+} http_tls_t;
+
+typedef union _cdsa_conn_ref_u		/**** CDSA Connection reference union
+					 **** used to resolve 64-bit casting
+					 **** warnings.
+					 ****/
+{
+  SSLConnectionRef connection;		/* SSL connection pointer */
+  int		   sock;		/* Socket */
+} cdsa_conn_ref_t;
 
 extern OSStatus	_httpReadCDSA(SSLConnectionRef connection, void *data,
 		              size_t *dataLength);
@@ -111,5 +124,5 @@ extern const char *hstrerror(int error);
 #endif /* !_CUPS_HTTP_PRIVATE_H_ */
 
 /*
- * End of "$Id: http-private.h 1439 2005-10-28 20:32:59Z mike $".
+ * End of "$Id: http-private.h 1493 2006-06-07 18:20:49Z mike $".
  */
