@@ -1,5 +1,5 @@
 /*
- * "$Id: image.h 1397 2005-04-24 19:20:32Z mike $"
+ * "$Id: image.h 1486 2006-05-31 15:27:28Z mike $"
  *
  *   Image management definitions for HTMLDOC, a HTML document processing
  *   program.
@@ -64,7 +64,8 @@ typedef struct			/**** Image structure ****/
  * Prototypes...
  */
 
-extern void	image_copy(const char *filename, const char *destpath);
+extern void	image_copy(const char *src, const char *realsrc,
+		           const char *destpath);
 extern image_t	*image_find(const char *filename, int load_data = 0);
 extern void	image_flush_cache(void);
 extern int	image_getlist(image_t ***ptrs);
@@ -78,5 +79,5 @@ extern void	image_unload(image_t *img);
 #endif /* !_IMAGE_H_ */
 
 /*
- * End of "$Id: image.h 1397 2005-04-24 19:20:32Z mike $".
+ * End of "$Id: image.h 1486 2006-05-31 15:27:28Z mike $".
  */
