@@ -1,5 +1,5 @@
 /*
- * "$Id: iso8859.cxx 1457 2005-11-03 16:27:45Z mike $"
+ * "$Id: iso8859.cxx 1479 2006-03-30 03:25:07Z mike $"
  *
  *   ISO-8859-1 conversion routines for HTMLDOC, an HTML document
  *   processing program.
@@ -356,6 +356,9 @@ iso8859(uchar *name)			/* I - Glyph name */
     char uniglyph[32];
     int newch;
 
+    if (!_htmlInitialized)
+      htmlSetCharSet("iso-8859-1");
+
     if ((glyph = _htmlGlyphsAll[ch]) == NULL)
     {
       sprintf(uniglyph, "uni%04x", ch);
@@ -379,6 +382,8 @@ iso8859(uchar *name)			/* I - Glyph name */
 	  else
 	    _htmlGlyphs[newch] = glyph;
 
+          _htmlUnicode[newch] = ch;
+
           // Reload font widths...
 	  htmlLoadFontWidths();
 
@@ -465,5 +470,5 @@ compare_lut(lut_t *a,	/* I - First glyph */
 
 
 /*
- * End of "$Id: iso8859.cxx 1457 2005-11-03 16:27:45Z mike $".
+ * End of "$Id: iso8859.cxx 1479 2006-03-30 03:25:07Z mike $".
  */
