@@ -1,10 +1,10 @@
 //
-// "$Id: license-oss.cxx,v 1.1.2.3 2004/10/23 06:16:41 mike Exp $"
+// "$Id: license.cxx 1439 2005-10-28 20:32:59Z mike $"
 //
 //   GUI license dialog routines for HTMLDOC, an HTML document processing
 //   program.
 //
-//   Copyright 1997-2004 by Easy Software Products.
+//   Copyright 1997-2005 by Easy Software Products.
 //
 //   These coded instructions, statements, and computer programs are the
 //   property of Easy Software Products and are protected by Federal
@@ -78,7 +78,7 @@ GUI::showLicenseCB(void)
   box = new Fl_Box(20, 45, 440, 110,
     "You are using the free, unsupported version of HTMLDOC which is "
     "provided under the terms of the GNU General Public License shown "
-    "below. HTMLDOC Basic and HTMLDOC Professional, which financially "
+    "below. Commercial versions of HTMLDOC, which financially "
     "support the continued development of HTMLDOC, may be purchased at the "
     "following URL:\n"
     "\n"
@@ -364,5 +364,5 @@ closeLicenseCB(Fl_Widget *w)		// I - Close button
 #endif // HAVE_LIBFLTK
 
 //
-// End of "$Id: license-oss.cxx,v 1.1.2.3 2004/10/23 06:16:41 mike Exp $".
+// End of "$Id: license.cxx 1439 2005-10-28 20:32:59Z mike $".
 //
