@@ -1,9 +1,9 @@
 /*
- * "$Id: config.h 1451 2005-10-31 21:55:06Z mike $"
+ * "$Id: config.h 1493 2006-06-07 18:20:49Z mike $"
  *
  *   Configuration file for HTMLDOC.
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: ESP Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: info@easysw.com
@@ -26,7 +26,7 @@
  * What is the version number for this software?
  */
 
-#define SVERSION	"1.8-current"
+#define SVERSION	"1.8.27"
 
 
 /*
@@ -54,8 +54,8 @@
  * Locations of files (overridden by the registry...)
  */
 
-#define DOCUMENTATION	"C:/Program Files/HTMLDOC/doc"
-#define HTML_DATA	"C:/Program Files/HTMLDOC"
+#define DOCUMENTATION	"C:/Program Files/Easy Software Products/HTMLDOC/doc"
+#define HTML_DATA	"C:/Program Files/Easy Software Products/HTMLDOC"
 
 
 /*
@@ -170,6 +170,6 @@
 
 
 /*
- * End of "$Id: config.h 1451 2005-10-31 21:55:06Z mike $".
+ * End of "$Id: config.h 1493 2006-06-07 18:20:49Z mike $".
  */
 
